//commhw.h
//Hardware interface for JVS Bridge
//Bryan E. Topp <betopp@betopp.com> 2020
#ifndef COMMHW_H
#define	COMMHW_H

#include <stdbool.h>
#include <stdint.h>

//Initializes communications hardware - UARTs and transmit-enable for 485.
void commhw_init();

//Checks for a byte on the PC-side UART.
//Returns true and stores the byte in *outbyte if one was available.
//Returns false and doesn't wait if no byte is available.
bool commhw_pcside_readb(uint8_t *outloc);

//Writes a byte on the PC-side UART.
//Waits before sending if the FIFO is full. 
//Returns when the byte has been written to the UART.
void commhw_pcside_writeb(uint8_t value);

//Checks for a byte on the JVS-side UART.
//Returns true and stores the byte in *outbyte if one was available.
//Returns false and doesn't wait if no byte is available.
//Requires that transmission be stopped before calling.
bool commhw_jvsside_readb(uint8_t *outloc);

//Writes a byte on the JVS-side UART.
//Waits before sending if the FIFO is full.
//Returns when the byte has been written to the UART.
void commhw_jvsside_writeb(uint8_t value);

//Enables or disables transmitter for JVS-side UART.
//If disabling, waits for UART transmission to finish first.
void commhw_jvsside_txenable(bool enabled);

//Sets the sense line on the JVS side.
//If the parameter is true, the sense line is driven to 0v (slave configured).
//If the parameter is false, the sense line is allowed to rise to 2.5v.
void commhw_jvsside_senselow(bool drivelow);

#endif	//COMMHW_H

