//jvsdefs.h
//Definitions for JVS controller input
//Bryan E. Topp <betopp@betopp.com> 2020
#ifndef JVSDEFS_H
#define JVSDEFS_H

//Sync character on JVS bus
#define JVS_BYTE_SYNC 0xE0

//Escape character on JVS bus
#define JVS_BYTE_ESC 0xD0

//JVS command codes
typedef enum jvs_cmd_e
{
	JVS_CMD_RESET = 0xF0, //Reset the communication status of all slaves to the initial state (Required)
	JVS_CMD_ADDRSET = 0xF1, //Set slave address (Required)
	JVS_CMD_CHCOM = 0xF2, //Change the communication speed of all nodes.
	
	JVS_CMD_IOID = 0x10, //Read ID data of each slave (Required)
	JVS_CMD_CMDREV = 0x11, //Read the command format revision (Required)
	JVS_CMD_JVSREV = 0x12, //Read the JAMMA VIDEO standard edition (revision) (Required)
	JVS_CMD_COMREV = 0x13, //Read the corresponding version of the communication method (Required)
	JVS_CMD_FNCHK = 0x14, //Load functions of each slave (Required)
	JVS_CMD_MAINID = 0x15, //Communicate main board ID to I / O
	
	JVS_CMD_INPUT_SW = 0x20, //Read the SW of the controller
	JVS_CMD_INPUT_COIN = 0x21, //Read the number of coins
	JVS_CMD_INPUT_ANALOG = 0x22, //Read analog input value
	JVS_CMD_INPUT_ROTARY = 0x23, //Read the rotary encoder value
	JVS_CMD_INPUT_KEYCODE = 0x24, //Load code
	JVS_CMD_INPUT_SCRPOS = 0x25, //Read the position on the screen such as touch panel, light gun, etc.
	JVS_CMD_INPUT_GPSW = 0x26, //SW input independent of player
		
	JVS_CMD_PAYOUTBAL = 0x2E, //Read information such as the number of remaining metal poppers
	JVS_CMD_RETRY = 0x2F, //Retransmission request in case of SUM error (Required)
	
	JVS_CMD_OUTPUT_COINSUB = 0x30, //Output coin subtraction number at game start
	JVS_CMD_OUTPUT_PAYOUTADD = 0x31, //Output number of payouts
	JVS_CMD_OUTPUT_GPDRV1 = 0x32, //Data output to parallel driver
	JVS_CMD_OUTPUT_ANALOG = 0x33, //Data output to analog output section
	JVS_CMD_OUTPUT_CHAR = 0x34, //ASCII code output to 7seg LED and LCD
	JVS_CMD_OUTPUT_COINADD = 0x35, //Command for adding coins
	JVS_CMD_OUTPUT_PAYOUTSUB = 0x36, //Commands for subtracting the number of payouts
	JVS_CMD_OUTPUT_GPDRV2 = 0x37, //Specific byte data output to parallel driver
	JVS_CMD_OUTPUT_GPDRV3 = 0x38, //Specific bit data output to parallel driver
	
} jvs_cmd_t;

//Status bytes - overall state of an ack packet from the slave
typedef enum jvs_status_e
{
	JVS_STATUS_NORMAL = 1,
	JVS_STATUS_BADCMD = 2, //Unsupported command
	JVS_STATUS_BADSUM = 3, //Checksum failure
	JVS_STATUS_ACKOVF = 4,
	
	JVS_STATUS_SILENT = 100, //Dummy value - don't send a reply packet.
} jvs_status_t;

//Report bytes - status of each report, per command issued
typedef enum jvs_report_e
{
	JVS_REPORT_NORMAL = 1,
	JVS_REPORT_BADPARM_NODATA = 2, //Bad parameter - no data returned from slave.
	JVS_REPORT_BADPARM_IGNORED = 3, //Bad parameter - data not accepted from master.
	JVS_REPORT_BUSY = 4,
} jvs_report_t;

//Player switches - first byte
typedef enum jvs_playersw0_e
{
	JVS_PLAYERSW0_PUSH2 = 0x01,
	JVS_PLAYERSW0_PUSH1 = 0x02,
	JVS_PLAYERSW0_RIGHT = 0x04,
	JVS_PLAYERSW0_LEFT  = 0x08,
	JVS_PLAYERSW0_DOWN  = 0x10,
	JVS_PLAYERSW0_UP    = 0x20,
	JVS_PLAYERSW0_SVC   = 0x40,
	JVS_PLAYERSW0_START = 0x80,
} jvs_playersw0_t;

//Player switches - second byte
typedef enum jvs_playersw1_e
{
	JVS_PLAYERSW1_PUSH10 = 0x01,
	JVS_PLAYERSW1_PUSH9  = 0x02,
	JVS_PLAYERSW1_PUSH8  = 0x04,
	JVS_PLAYERSW1_PUSH7  = 0x08,
	JVS_PLAYERSW1_PUSH6  = 0x10,
	JVS_PLAYERSW1_PUSH5  = 0x20,
	JVS_PLAYERSW1_PUSH4  = 0x40,
	JVS_PLAYERSW1_PUSH3  = 0x80,
} jvs_playersw1_t;

#endif //JVSDEFS_H
