//JVS-Bridge Public Release: Copyright 2022 Tucson Logic LLC.
//This program may be distributed under the terms of the GNU GPL v3.
//See the LICENSE file for more information.

//This file is derived from a SiLabs example project.

/**************************************************************************//**
 * @file
 * @brief   Callbacks and ISR's.
 * @author  Silicon Laboratories
 * @version 1.0.0 (DM: July 14, 2014)
 *
 *******************************************************************************
 * @section License
 * (C) Copyright 2014 Silicon Labs Inc,
 * http://developer.silabs.com/legal/version/v11/Silicon_Labs_Software_License_Agreement.txt
 *******************************************************************************
 *
 * Definitions and external declarations for VCPXpress_callback.c
 * We also place the UART and TIMER ISR here.
 * 
 *****************************************************************************/

#include <SI_EFM8UB2_Register_Enums.h>                // SI_SFR declarations
#include "buffers.h"
#include "initialization.h"
#include "uart_0.h"

//Here we say if we are idle for 10 flush.
#define RX_FLUSH_TIME 10   //<! Number of milliseconds the uart needs 
                           //   to be idle before we flush whatever
                           //   data we have.

/**************************************************************************//**
 *  @brief
 *    VCPXpress callback
 *    
 *  Called by VCPXpresss. We only need to handle TX/RX compelte and device_open
 *   
 *****************************************************************************/
void VCP_Callback(void)
{
   uint32_t INTVAL = Get_Callback_Source();

   // Device was opened so reset
   if (INTVAL & DEVICE_OPEN)
   {

     //Reset variable state
     resetState();

   }// if DEVICE_OPEN

   // Rx compete ready for next RX buffer.
   if (INTVAL & RX_COMPLETE)
   {
  	 if(*usbRxLen != 0)
  	 {
  		 //if we received a packet of non-zero length switch to next
  		 // buffer. *usbRxLen is already set by usb library.
       usbSwitchRxbuffer();
  	 }

  	 // Flag main that next transfer can start
  	 // in the case of a zlp this will restart the receive.
  	 usbRxReady = 1;

   }// if RX_COMPLETE

   // Rx compete ready for next TX buffer.
   if (INTVAL & TX_COMPLETE)
   {
  	 //Only switch buffer if we completed a packet of non-zero length
  	 if(*usbTxLen != 0)
  	 {
    	 // When TX complete perform buffer switch
       *usbTxLen = 0;       // mark current buffer empty now that TX is done
       usbSwitchTxbuffer(); // switch usb buffer
  	 }

  	 // Flag main while loop that it can send data up
  	 usbTxReady = 1;
   
   }// if TX_COMPLETE
}//VCPXpress_API_CALLBACK()




