//JVS-Bridge Public Release: Copyright 2022 Tucson Logic LLC.
//This program may be distributed under the terms of the GNU GPL v3.
//See the LICENSE file for more information.

//commhw.c
//Hardware interface for JVS Bridge
//Bryan E. Topp <betopp@betopp.com> 2020

#include <SI_EFM8UB2_Register_Enums.h>                // SI_SFR declarations
#include "buffers.h"

#include "commhw.h"

SI_SEGMENT_VARIABLE(commhw_pcside_bufferpos, uint16_t, SI_SEG_XDATA);

void commhw_init()
{    
	//Port 0 totally digital
	P0MDIN = 0xFF;

	//Pins 4 (TX) and 6 (direction) push-pull
	P0MDOUT = 0x50;
	P0 = 0xFF;

	//Start out looking like we've finished a transmission (idle)
	SBUF0 = 0xFF;
}

bool commhw_pcside_readb(uint8_t *outloc)
{
	//return false;

	//We're working from the USB-UART example.
	//In that scenario, data received on USB goes into a "UART TX" buffer.
	//So our notion of reading from the PC is to pull from the "UART TX" side.
	if(commhw_pcside_bufferpos < *uartTxLen)
	{
		*outloc = uartTxBuf[commhw_pcside_bufferpos];
		commhw_pcside_bufferpos++;
		return true;
	}
	else
	{
		*uartTxLen = 0;
		switchTxBuffer();
		commhw_pcside_bufferpos = 0;
		return false;
	}
}

void commhw_pcside_writeb(uint8_t value)
{
	(void)value;
	return;
}

bool commhw_jvsside_readb(uint8_t *outloc)
{
	if(SCON0_RI)
	{
		SCON0_RI = 0;
		*outloc = (uint8_t)(SBUF0);
		return true;
	}
	else
	{
		return false;
	}
}

void commhw_jvsside_writeb(uint8_t value)
{
	  while(!SCON0_TI) { }
	  SBUF0 = value;
	  SCON0_TI = 0;
}

void commhw_jvsside_txenable(bool enabled)
{
    if(enabled)
    {
        P0 |= (1<<6);
    }
    else
    {
        //Wait for transmission to complete before turning off transmitter
    	while(!SCON0_TI) { }
        P0 &= ~(1<<6);
    }
}

void commhw_jvsside_senselow(bool drivelow)
{
    if(drivelow)
        P0 &= ~(1<<3);
    else
        P0 |= (1<<3);
}
