//JVS-Bridge Public Release: Copyright 2022 Tucson Logic LLC.
//This program may be distributed under the terms of the GNU GPL v3.
//See the LICENSE file for more information.

//This ugly main.c is derived from a SiLabs example project.
//This was mainly to get the USB UART stuff working correctly.
// -bet

/**************************************************************************//**
 * (C) Copyright 2014 Silicon Labs Inc,
 * http://developer.silabs.com/legal/version/v11/Silicon_Labs_Software_License_Agreement.txt
 * 
 *****************************************************************************/

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <SI_EFM8UB2_Register_Enums.h>                // SI_SFR declarations
#include "initialization.h"
#include "buffers.h"
#include "uart_0.h"

#include "commhw.h"
#include "commands.h"
#include "coinage.h"
#include "jvs.h"
#include "analog.h"

//-----------------------------------------------------------------------------
// SiLabs_Startup() Routine
// ----------------------------------------------------------------------------
// This function is called immediately after reset, before the initialization
// code is run in SILABS_STARTUP.A51 (which runs before main() ). This is a
// useful place to disable the watchdog timer, which is enable by default
// and may trigger before main() in some instances.
//-----------------------------------------------------------------------------
void SiLabs_Startup (void)
{
   PCA0MD &= ~0x40;                               
}
 

int main (void)
{  
   uint8_t result = VCP_STATUS_OK;
   uint8_t zero = 0;

   // Disable watchdog timer
   

   System_Init ();   // Call top-level initialization routine
   resetState();     // reset buffer statues and init uart
   IE_EA = 1;        // enable global interrupts

   VCP_Set_Suspend(&zero);

   commhw_init();
   commands_init();
   jvsslave_init();
   analog_init();

   //Main Thread. Here we monitor the current buffer for each
   //  transfer and kick the transfer when appropriate.
   while(1)
   {

     // if usb is ready for the next Tx packet and the current Tx
     // buffer has data then send the current buffer.
     if(usbTxReady && *usbTxLen){



    	 //VCPXpress library is not reentrant so we disable
       //  the USB interrupt to ensure it doesn't fire during
       //  the block write call
       EIE1 &= ~EIE1_EUSB0__BMASK;
       usbTxReady = 0;
       result = Block_Write(usbTxBuf, *usbTxLen, usbTxLen);
       EIE1 |= EIE1_EUSB0__BMASK;

       //If our write request failed retry
       if(result != VCP_STATUS_OK)
       {
      	 usbTxReady = 1;
       }

       //Buffer will be switched by callback when transfer completes

     }
      
      //If USB receive has completed and the current buffer
      // is empty. Prime for next receive.
      if(usbRxReady && !*usbRxLen)
      {
        //VCPXpress library is not reentrant so we disable
        //  the USB interrupt to ensure it doesn't fire during
        //  the block read call
        EIE1 &= ~EIE1_EUSB0__BMASK;
        usbRxReady = 0;
        Block_Read(usbRxBuf, USB_BLOCK_SIZE, usbRxLen);
        EIE1 |= EIE1_EUSB0__BMASK;

        //If our read request failed retry
        if(result != VCP_STATUS_OK)
        {
        	usbRxReady = 1;
        }

        //Buffer will be switched by callback when transfer completes.

      }// If usb ready for next receive

      jvsslave_update();
      commands_update();
      coinage_update();
      analog_update();

   }// while(forever)
}//main()
